#include "scriptengine.h"
#include "main.h"

CScriptEngine eng;

void main() 
{
	CItem *a;
	CItem *b;
	CPlayer *p;

	asDWORD retVal;

	// Init the script engine and load our script
	if (!eng.init()) return;
	if (!eng.loadScript("script.as")) return;

	// Registers item types
	eng.callScript("item_init", 0x0, &retVal);

	if (!retVal) {
		printf("Could not initialise items\n");
		return;
	}

	// Create some objects to play with
	a = new CItem("mine");
	b = new CItem("health");
	p = new CPlayer;

	p->health = 100;

	// the collidePlayer call is dispatched to the appropriate script function
	a->collidePlayer(p);
	printf("Health: %.2f\n", p->health);
	a->collidePlayer(p);
	printf("Health: %.2f\n", p->health);
	b->collidePlayer(p);
	printf("Health: %.2f\n", p->health);

	delete a;
	delete b;
	delete p;
	return;
}

vector<string> CItem::itemNames; // Static class members must have a presence outside the definition

CItem::CItem(string type) {
	string func;
	list<asDWORD *> args;

	CItem *thisPtr;

	vector<string>::iterator it = find(itemNames.begin(), itemNames.end(), type);
	// Ensure that this is a valid type
	if (it != itemNames.end())
	{
		itemType = type;
		thisPtr = this;

		func = type;	// Function to call in angelscript. Constructor = type name.
		args.push_back((asDWORD *)&thisPtr);

		eng.callScript(func, &args);
		return;
	}

	printf("Error: Tried to create an item that doesn't exist\n");
}

void CItem::registerFloat(const char *varname) {
	string temp = varname;
	vars.insert(pair<string,float>(temp, 0.0));
}

void CItem::setFloat(const char *varname, float value) {
	string temp = varname;
	vars[temp] = value;
}

float CItem::getFloat(const char *varname) {
	string temp = varname;
	return vars[temp];
}

void CItem::collidePlayer(CPlayer *p) {
	string func;
	list<asDWORD *> args;

	CItem *thisPtr;
	
	thisPtr = this;
	func = itemType + "_collidePlayer";	// Function to call. This could be optimised by storing the function ID.
	
	args.push_back((asDWORD *)&thisPtr);
	args.push_back((asDWORD *)&p);
	eng.callScript(func, &args);
}

void CItem::registerItemType(const char *itemName) {
	CItem::itemNames.push_back(itemName);
}